"""
create:2018/10/23
by:yhleng
"""
import time
import os
import string
import random
import zipfile
import csv
from selenium import webdriver
import yaml
from aceui.lib import gl
from aceui.lib.config import CONF


def select_Browser_WebDriver():
    """
    根据config.yaml配置文件，来选择启动的浏览器
    :return:
    """
    #读取config.yam配置文件中，浏览器配置
    bro_name = CONF.read(gl.configFile)['CONFIG']['Browser']

    #根据borName决定，启动，哪个浏览器
    if str(bro_name).strip().lower() == 'chrome':
        driver = webdriver.Chrome()
    elif str(bro_name).strip().lower() == 'firefox':
        driver = webdriver.Firefox()
    else:
        driver = webdriver.Ie()
    return driver



def write_ymal(path, data):
    """
    写YAML文件内容
    :param path: YAML文件路径
    :param data: 写入的数据
    :return: 无
    """
    with open(path, 'wb') as fp:
        yaml.dump(data, fp)


def get_yaml_field(path):
    """
    读取YAML内容
    :param path: xxxx.YAML文件所在路径
    :return: 指定节点内容
    """
    with open(path, 'rb') as fp:
        cont = fp.read()

    ret = yaml.load(cont, Loader=yaml.FullLoader)
    return ret


def get_run_flag(scenario_key, casename):
    """
    获取运行标记，来决定是否执行
    :param scenario_key:
    :return: Y 或 N
    """
    conf = CONF.read(gl.configFile)
    return conf['RUNING'][scenario_key]['Flag'][casename]['Flag']



def cook_info(func):
    """
    从配置文件获取cookies信息
    :param func: 函数名
    :return: 函数
    """
    def warpper(*args, **kwargs):
        cook1= CONF.read(gl.configFile)['CONFIG']['Cookies']['LoginCookies']
        return func(cook=cook1, *args, **kwargs)
    return warpper


def replay(func):
    """
    回放速度
    :param func: 函数名
    :return: wrapper
    """
    def wrapper(*args, **kwargs):
        func(*args, **kwargs)
        sleeptime = float(CONF.read(gl.configFile)['RUNING']['REPLAY']['Time']) / 1000
        time.sleep(sleeptime)
        return func
    return wrapper



def hight_light_conf(key):
    """
    配置元素，是否高亮显示
    :param key: config.yaml 中关键字 HightLight:1 高亮 其它忽略
    :return:
    """
    def _wrapper(func):
        def wrapper(*args, **kwargs):
            ret = None
            if CONF.read(gl.configFile)[key] == 1:
                ret = func(*args, **kwargs)
            return ret
        return wrapper
    return _wrapper



def remove_all_files(dirpath):
    """
    删除目标,目录下文件及文件夹
    :param dirpath: 目标目录
    :return: 无
    """
    listdir = os.listdir(dirpath)
    if listdir:
        for f in listdir:
            filepath = os.path.join(dirpath, f)
            if os.path.isfile(filepath):
                os.remove(filepath)
            if os.path.isdir(filepath):
                os.rmdir(filepath)


def zip_dir(dirpath,out_fullName):
    """
    压缩指定文件夹
    :param dirpath: 目标文件夹路径
    :param outFullName: 压缩文件保存路径+xxxx.zip
    :return: 无
    """
    zip = zipfile.ZipFile(out_fullName, "w", zipfile.ZIP_DEFLATED)
    for path, dirnames, filenames in os.walk(dirpath):
        # 去掉目标跟路径，只对目标文件夹下边的文件及文件夹进行压缩
        fpath = path.replace(dirpath, '')

        for filename in filenames:
            zip.write(
                os.path.join(path, filename),
                os.path.join(fpath, filename)
            )
    zip.close()


def reply_case_fail(num=3):
    """
    测试case失败后，重新执行功能
    :param num: 失败最多可以执行次数，默认为3次
    :return: fun本身或者抛出异常
    """
    def _warpper(func):
        def warpper(*args, **kwargs):
            raise_info = None
            rnum = 0
            num = CONF.read(gl.configFile)['RUNING']['CASE']['Retry']
            for _ in range(num):
                try:
                    ret = func(*args, **kwargs)
                    if rnum > 1:
                        print('重试{}次成功'.format(rnum))
                    return ret
                except Exception as ex:
                    rnum += 1
                    raise_info = ex
            print('重试{}次,全部失败'.format(rnum))
            raise raise_info
        return warpper
    return _warpper


def get_data(file, field):
    """
    从data目录下的yaml文件读取指定字段(field)CASE数据
    :param file:yaml数据文件
    :param field: 指定字段读取
    :return: 结构数据
    """
    yaml = (str(file).endswith('.yaml'), str(file).endswith('.yam'))
    file = '{}.yaml'.format(file) if not (yaml[0] or yaml[1]) else file
        

    field = str(field).strip().upper()
    data = CONF.read(
        os.path.join(
            gl.dataPath, file
        )
    )[field]
    return data


def create_dir(path):
    """
    如果文件夹不存在，则创建
    :param path:
    :return:
    """
    folder = os.path.exists(path)
    if not folder:
        os.makedirs(path)


def genrandomstr(lenstr=5):
    """
    随机产生数据
    :param lenstr: 需要字符串的长度,
    :return: 指定长度的字符和数字组合的字符串，例如：lenstr=5，"Qamo8"
    """
    strs = "".join(random.choice(string.ascii_letters + string.digits)
                   for _ in range(lenstr)
                   )
    return strs


def rndint(min=0, max=1):
    """
    返回一个随机整数
    :param max:
    :param min:
    :return:
    """
    return random.randint(min, int(max))


def join_url(url):
    """
    读取config基本url并拼接完整
    :param url: 页面url
    :return: 完整url
    """
    base_url = CONF.read(gl.configFile)['CONFIG']['Base_Url']

    if not url.startswith('/'):
        url = "/{}".format(url)

    return "{}{}".format(base_url, url)


def autoi(strword):
    """
    将字符串类型的数字转换成int类型
    :param strword: 字符串数字，'1,989.00'
    :return: int类型 1989
    """
    if strword.count('0') != 0 or strword.count(',') != 0:
        strword = ''.join(
            strword.split('.')[0].split(',')
        )
    
    return int(strword)



def rnd_num(len=5):
    """
    返回一个随机整数
    :param max:
    :param min:
    :return:
    """
    rnd = ''
    for _ in range(len):
        rnd += str(random.randint(0, 9))

    return rnd



def createphone():
    """
    :return: 手机号
    """
    mobile_list = ['139', '138', '136', '133',
               '134', '156', '158', '155', '159'
               ]
    phone = '{}{}'.format(
        random.choice(mobile_list),
        ''.join(random.choice('0123456789') for _ in range(8))
    )

    return phone


def write_csvfile(filepath, header, data):
    """
    :param filepath:csv文件路径
    :param header:csv文件标头（表格第一行）[]
    :param data:写入数据，[{}]
    :return:无
    """
    with open(filepath, 'w', newline='') as files:
        writer = csv.DictWriter(files, header)
        writer.writeheader()
        writer.writerows(data)


def send_dding_msg(token, msg):
    '''
    推送消息到钉群
    '''
    import requests
    payload = {
        "msgtype": "text",
        "text": {
            "content": msg,
            # "mentioned_mobile_list":["13718651998"],
        }
    }
    headers = {
        'Content-Type': 'application/json'
    }
    d_url = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={}'.format(token)

    res = requests.request('POST', d_url, headers=headers, json=payload)

    return res


if __name__ == "__main__":
    # stra = genrandomstr(20)
    # print(type(stra))
    # url = join_url('/activity/create/1024')
    print(createphone())





"""
__doc__:
create:2018/10/27
by: yhleng
"""
import os

import time
from PIL import Image
from selenium.common.exceptions import (
    NoSuchElementException,
    ElementNotVisibleException,
    UnexpectedAlertPresentException
)
from selenium.webdriver.support.select import Select
from aceui.lib import gl
from aceui.lib.scripts import (
    get_yaml_field,
    replay,
    hight_light_conf,
    reply_case_fail,
    genrandomstr,
    rndint,
    rnd_num,
    get_data
)
from selenium.webdriver.common.action_chains import ActionChains

class BasePage:
    """PO公共方法类"""
    def __init__(self, baseurl, driver, pagetitle):
        """
        初始化,driver对象及数据
        :param baseurl: 目标地址
        :param driver: webdriver对象
        :param pagetitle: 用来断言的目标页标题
        """
        self.base_url = baseurl
        self.driver = driver
        self.pagetitle = pagetitle



    #－－－－－－－－－－－－－－－－－
    # 功能描述：所有公共方法，都写在以下
    # －－－－－－－－－－－－－－－－－

    #打开浏览器
    def _open(self, url):
        """
        打开浏览器，并断言标题
        :param url: 目标地址
        :return: 无
        """
        self.driver.maximize_window()
        self.driver.get(url)

        self.driver.implicitly_wait(10)
        self.get_image
        self.driver.implicitly_wait(0)


    def is_display_timeout(self, element, timeses, *loc):
        """
        在指定时间内，轮询元素是否显示
        :param element: 元素对象
        :param timeSes: 轮询时间
        :return: bool
        """
        start_time = int(time.time()) #秒级时间戳
        timeses = int(timeses)
        while (int(time.time())-start_time) <= timeses:
            if element.is_displayed() and element.is_enabled():
                return True
            self.wait(500)
            element = self.driver.find_element(*loc)
        self.get_image
        return False

    @reply_case_fail(num=3)
    def find_element(self, *loc):
        """
        在指定时间内，查找元素；否则抛出异常
        :param loc: 定位器
        :return: 元素 或 抛出异常
        """
        timeout = 10
        try:
            self.driver.implicitly_wait(timeout) #智能等待；超时设置

            element = self.driver.find_element(*loc) #如果element没有找到，到此处会开始等待
            if self.is_display_timeout(element, timeout, *loc):
                self.hightlight(element)  #高亮显示
                self.driver.implicitly_wait(0)  # 恢复超时设置
            else:
                raise ElementNotVisibleException #抛出异常，给except捕获
            return element
        except (NoSuchElementException, ElementNotVisibleException) as ex:
            self.get_image
            raise ex
        else:
            self.get_image


    @hight_light_conf('HightLight')
    def hightlight(self, element):
        """
        元素高亮显示
        :param element: 元素对象
        :return: 无
        """
        # arguments[0] 为element
        # arguments[1]为"border: 2px solid red;"
        self.driver.execute_script(
            "arguments[0].setAttribute('style', arguments[1]);",
            element,
            "border: 2px solid red;" #边框border:2px; red红色
        )


    def get_element_image(self, element):
        """
        截图,指定元素图片
        :param element: 元素对象
        :return: 无
        """
        timestrmap = time.strftime('%Y%m%d_%H.%M.%S')
        imgpath = os.path.join(
            gl.imgPath,
            '%s.png' % str(timestrmap)
        )

        #截图，获取元素坐标
        self.driver.save_screenshot(imgpath)
        left = element.location['x']
        top = element.location['y']
        e_width = left + element.size['width']
        e_height = top + element.size['height']

        picture = Image.open(imgpath)
        picture = picture.crop(
            (
                left,
                top,
                e_width,
                e_height
            )
        )
        timestrmap = time.strftime('%Y%m%d_%H.%M.%S')
        imgpath = os.path.join(
            gl.imgPath, '%s.png' % str(timestrmap)
        )
        picture.save(imgpath)
        print('screenshot:', timestrmap, '.png')


    @property
    def get_image(self):
        '''
        截取图片,并保存在images文件夹
        :return: 无
        '''
        timestrmap = time.strftime('%Y%m%d_%H.%M.%S')
        imgpath = os.path.join(
            gl.imgPath,
            '%s.png' % str(timestrmap)
        )

        self.driver.save_screenshot(imgpath)
        print('screenshot:', timestrmap, '.png')


    @reply_case_fail(num=3)
    def find_elements(self, *loc):
        '''批量找标签'''
        timeout = 10 #智能等待时间
        try:
            self.driver.implicitly_wait(timeout) #智能等待；此贯穿self.driver整个生命周期
            elements = self.driver.find_elements(*loc)

            self.driver.implicitly_wait(0) #恢复等待
            return elements

        except NoSuchElementException as ex:
            self.get_image  # 截取图片
            raise ex


    def iter_click(self, *loc):
        '''批量点击某元素'''
        element = self.find_elements(*loc)
        for e in element:
            e.click()


    def iter_input(self, text=None, *loc):
        """
        批量输入
        :param text: 输入内容;为list
        :param loc: 定位器(By.XPATH,'//*[@id='xxxx']/input')
        :return: 无
        """
        elements = self.find_elements(*loc)
        for i, e in enumerate(elements):
            self.wait(1000)
            #e.clear()
            e.send_keys(list(text)[i])


    #文本框输入
    def send_keys(self, content, *loc):
        '''
        :param content: 文本内容
        :param itype: 如果等1，先清空文本框再输入。否则不清空直接输入
        :param loc: 文本框location定位器
        :return:
        '''
        inputelement = self.find_element(*loc)
        #inputElement.clear()
        inputelement.send_keys(content)



    def clear_input_text(self, *loc):
        '''清除文本框内容'''
        self.find_element(*loc).clear()



    def add_cookies(self, ck_dict):
        '''
        增加cookies到浏览器
        :param ck_dict: cookies字典对象
        :return: 无
        '''
        for key in ck_dict.keys():
            self.driver.add_cookie(
                {
                    "name":key,
                    "value":ck_dict[key]
                }
            )

    def is_display(self, *loc):
        #isDisable
        '''
        元素存在,判断是否显示
        :param loc: 定位器
        :return: 元素存在并显示返回True;否则返回False
        '''
        timeout = 20
        try:
            self.driver.implicitly_wait(timeout)

            element = self.driver.find_element(*loc)
            if self.is_display_timeout(element, timeout, *loc):
                self.hightlight(element)
                return True
            else:
                return False
            self.driver.implicitly_wait(0)
        except (
                NoSuchElementException,
                ElementNotVisibleException,
                UnexpectedAlertPresentException) as ex:
            self.get_image #10秒还未找到显示的元素
            return False



    def is_exist(self, *loc):
        """
        判断元素,是否存在
        :param loc: 定位器(By.ID,'kw')
        :return: True 或 False
        """
        timeout = 60
        try:
            self.driver.implicitly_wait(timeout)
            e = self.driver.find_element(*loc)

            """高亮显示,定位元素"""
            self.hightlight(e)

            self.driver.implicitly_wait(0)
            return True
        except NoSuchElementException as ex:
            self.get_image #10秒还未找到元素，截图
            return False


    def exist_and_click(self, *loc):
        '''如果元素存在则单击,不存在则忽略'''
        print('Click:{0}'.format(loc))

        timeout = 3 #超时 时间
        try:
            self.driver.implicitly_wait(timeout)

            element = self.driver.find_element(*loc)
            self.hightlight(element)
            element.click()
            self.driver.implicitly_wait(0)
        except NoSuchElementException as ex:
            pass

    def exist_and_input(self, text, *loc):
        '''如果元素存在则输入,不存在则忽略'''
        print('Input:{0}'.format(text))

        timeout = 3
        try:
            self.driver.implicitly_wait(timeout)

            element = self.driver.find_element(*loc)
            self.hightlight(element) #高亮显示
            element.send_keys(str(text).strip())

            self.driver.implicitly_wait(0)
        except (NoSuchElementException, ElementNotVisibleException) as ex:
            pass


    def get_tag_text(self, txt_name, *loc):
        """
        获取元素对象属性值
        :param propertyName: Text属性名称
        :param loc: #定位器
        :return: 属性值 或 raise
        """
        timeout = 20
        try:
            self.driver.implicitly_wait(timeout)

            element = self.find_element(*loc)
            self.hightlight(element) #高亮显示

            #获取属性
            pro_value = getattr(element, str(txt_name))
            self.driver.implicitly_wait(0)
            return pro_value
        except (NoSuchElementException, NameError) as ex:
            self.get_image #错误截图
            raise ex




    @property
    def switch_window(self):
        """
        切换window窗口,切换一次后退出
        :return: 无
        """
        cur_handle = self.driver.current_window_handle
        all_handle = self.driver.window_handles
        for h in all_handle:
            if h != cur_handle:
                self.driver.switch_to.window(h)
                break



    def wait(self, ms):
        """
        线程休眼时间
        :param ms: 毫秒
        :return: 无
        """
        ms = float(ms) / 1000
        time.sleep(ms)

    @replay
    def js_click(self, desc, *loc):
        """通过js注入的方式去，单击元素"""
        print('Click{}:{}'.format(desc, loc))
        element = self.find_element(*loc)
        self.driver.execute_script(
            "arguments[0].click();",
            element
        )


    @replay
    def input_text(self, text, desc, *loc):
        """
        输入文本操作
        :param text:
        输入文本内容;如果为%BLANK%，为清除输入框默认值;
        输入文本内容；如果为%NONE%,为不做任何操作
        :param desc:
        :param loc:
        :return:
        """
        print('Input{}:{}'.format(desc, text))
        var = get_data(gl.configFile,'CONFIG')
        flag_conf = var['Custom_Var']
        # 判断是自定义函数%%还是普通文本
        if str(text).startswith('%') and str(text).endswith('%'):
            flag = str(text)
            #判断是自定义函数是否带参数
            if ('(' in str(text)) and (')' in str(text)):

                s = str(text).rsplit('(')
                flag = '{}{}'.format(s[0],'%')

                param = s[1].rsplit(')')[0]
                #eval恢复函数原型并调用
                eval(str(flag_conf[flag]).format(param))
            else:
                flag = str(text)
                eval(flag_conf[flag])
        else:
            self.send_keys(text, *loc)



    @replay
    def input_text_index(self, desc, text ,index, *loc):
        """
        按索引输入文本
        :param desc: 输入框描述
        :param text: 输入内容
        :param index: 元素索引
        :param loc: 定位器
        :return: 无
        """
        print('Input{}:{}'.format(desc, text))
        index = int(index)
        var = get_data(gl.configFile,'CONFIG')
        flag_conf = var['Custom_Var']
        # 判断是自定义函数%%还是普通文本
        if str(text).startswith('%') and str(text).endswith('%'):
            flag = str(text)
            #判断是自定义函数是否带参数
            if ('(' in str(text)) and (')' in str(text)):

                s = str(text).rsplit('(')
                flag = '{}{}'.format(s[0],'%')

                param = s[1].rsplit(')')[0]
                #eval恢复函数原型并调用
                eval(str(flag_conf[flag]).format(param))
            else:
                flag = str(text)
                eval(flag_conf[flag])
        else:
            ele = self.find_elements(*loc)[index]
            ele.send_keys(text)


    @replay
    def click_button(self, desc, *loc):
        """点击操作"""
        print('Click:{}{}'.format(desc, loc))
        if str(desc).strip().upper() == '%NONE%':
            pass
        else:
            ele = self.find_element(*loc)
            self.action_chains.move_to_element(ele).move_by_offset(5,5).click().perform()


    @replay
    def click_btn_index(self, desc, index, *loc):
        """
        点击操作，按索引，适用于findelemens方法
        :param desc: 描述
        :param index: 点击索引
        :param op: 如果是True则，不执行点击操作，为False则点击
        :param loc: 定位器
        :return: 无
        """
        print('Clicks:{}{}'.format(desc, loc))
        if index == '%NONE%':
            pass
        else:
            ele = self.find_elements(*loc)[int(index)]
            # 元素高亮显示
            self.hightlight(ele)
            # 元素单击
            self.action_chains.move_to_element(ele).move_by_offset(5,5).click().perform()


    @replay
    def select_tab(self, *loc):
        '''选择tab操作'''
        print('Select:{}'.format(loc))
        self.find_element(*loc).click()


    @property
    def open(self):
        """打开浏览器，写入cookies登录信息"""
        yamldict = get_yaml_field(gl.configFile)
        ck_dict = yamldict['CONFIG']['Cookies']['LoginCookies']
        self._open(self.base_url)
        self.add_cookies(ck_dict)
        self._open(self.base_url)
        assert self.driver.title == self.pagetitle, "断言标题错误,请查检页面"


    def execute_script(self, js):
        """执行js脚本"""
        self.driver.execute_script(js)


    def select_list(self, *loc):
        """
        创建Select对象
        :param loc: 定位
        :return: Select对象
        """
        st = Select(
            self.find_element(*loc)
        )
        return st



    def get_element_attribute(self, attr, *loc):
        """
        获取元素属性
        :param attr: 属性
        :param loc: 定位
        :return: 无
        """
        ele = self.find_element(*loc)
        try:
            att = ele.get_attribute(attr)
        except Exception as ex:
            print('属性错误:{}'.format(attr))
            raise ex
        return att


    def set_element_attribute(self,attr, val ,*loc):
        """
        设置元素属性值,适用于元素唯一
        :param attr: 元素属性名称如:class
        :param val: 元素属性值
        :param loc: 定位
        :return: 无
        """
        element = self.find_element(*loc)
        self.driver.execute_script(
            "arguments[0].attr({},{});".format(attr, val),
            element
        )

    def del_element_attribute(self, attr, *loc):

        element = self.find_element(*loc)
        self.driver.execute_script(
            "arguments[0].removeAttribute({});".format(attr),
            element
        )

    def set_elements_attribute(self,attr, val ,index ,*loc):
        """
        设置元素属性
        :param attr: 元素属性名如:class
        :param val: 元素属性值
        :param index: 元素索引,为1000时代表全部设置
        :param loc: 元素定位
        :return: 无
        """
        index = int(index)
        element = self.find_elements(*loc)
        if index == 1000:
            for ele in element:
                self.driver.execute_script(
                    "arguments[0].attr({},{});".format(attr, val),
                    ele
                )
        else:
            self.driver.execute_script(
                "arguments[0].attr({},{});".format(attr, val),
                element[index]
            )


    def get_index_text(self, txt_name, index, *loc):
        """
        获取元素对象属性值
        :param propertyName: Text属性名称
        :param loc: #定位器
        :return: 属性值 或 raise
        """
        timeout = 10
        try:
            self.driver.implicitly_wait(timeout)

            element = self.find_elements(*loc)[int(index)]
            self.hightlight(element) #高亮显示

            #获取属性
            pro_value = getattr(element, str(txt_name))
            self.driver.implicitly_wait(0)
            return pro_value
        except (NoSuchElementException, NameError) as ex:
            self.get_image #错误截图
            raise ex


    def get_attribute_index(self, attr, index,*loc):
        """
        获取元素属性
        :param attr: 属性
        :param loc: 定位
        :return: 无
        """
        ele = self.find_elements(*loc)[int(index)]
        try:
            att = ele.get_attribute(attr)
        except Exception as ex:
            print('属性错误:{}'.format(attr))
            raise ex
        return att

    @property
    def action_chains(self):
        """右键方法"""
        action = ActionChains(self.driver)
        return action


if __name__ == "__main__":
    base = BasePage('', '', '')
    base.input_text('%RND(5)%', '', '')



import yaml
from aceui.lib import gl


class CONF:

    """写YAML文件内容"""
    @staticmethod
    def write(data):
        """
        写YAML文件内容
        :param path: YAML文件路径
        :param data: 写入的数据
        :return: 无
        """
        with open(gl.configFile,'wb') as fp:
            yaml.dump(data, fp)
            fp.close()

    """读取YAML文件内容"""
    @staticmethod
    def read(file):
        """
        读取YAML内容
        :return: 指定节点内容
        """
        with open(file,'rb') as fp:
            content = fp.read().decode('utf-8')
            fp.close()

        ret = yaml.load(content, Loader=yaml.FullLoader)
        return ret

# 单例模式
CONF()


"""
__doc__:
email 此模块用来，发送报告及邮件，格式为定制
如果要用在其它地方，需要修改
create:2018/10/23
by:yhleng
"""
import os
import time
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.header import Header
from email import encoders
from aceui.lib import (gl, scripts)


class EmailClass(object):
    """发送电子邮件"""
    def __init__(self):
        self.cur_date_time = str(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())) #当前日期时间
        self.config = scripts.get_yaml_field(gl.configFile) #配置文件路径
        self.sender = self.config['EMAIL']['Smtp_Sender'] # 从配置文件获取，发件人
        self.receivers = self.config['EMAIL']['Receivers']  # 从配置文件获取，接收人
        self.msg_title = self.config['EMAIL']['Msg_Title'] #从配置文件获取，邮件标题
        self.sender_server = self.config['EMAIL']['Smtp_Server'] #从配置文件获取，发送服务器
        self.From = self.config['EMAIL']['From']
        self.To = self.config['EMAIL']['To']


    @property
    def setMailContent(self):
        """配置邮件内容"""
        print(self.receivers)
        msg = MIMEMultipart()
        msg['From'] = Header(self.From, 'utf-8')
        msg['To'] = self.To
        msg['Subject'] = Header('%s%s'%(self.msg_title, self.cur_date_time), 'utf-8')

        #附件路径
        dirpath = gl.reportPath
        zipfile = os.path.join(os.path.dirname(dirpath), 'report.zip')
        reportfile = os.path.join(gl.reportPath, 'Report.html')
        scripts.zip_dir(dirpath, zipfile) #压缩报告
        #增加邮件内容为html
        fp = open(reportfile, 'rb')
        reportHtmlText = fp.read()
        msg.attach(MIMEText(reportHtmlText, 'html', 'utf-8'))
        fp.close()

        #增加附件
        html = self.addAttach(zipfile, filename='Report%s.zip'%self.cur_date_time) #自动化测试报告附件
        msg.attach(html)

        return msg



    def addAttach(self,apath, filename='Report.html'):
        """增加邮件"""
        with open(apath, 'rb') as fp:
            attach = MIMEBase('application', 'octet-stream')
            attach.set_payload(fp.read())
            attach.add_header('Content-Disposition', 'attachment', filename=filename)
            encoders.encode_base64(attach)
            return attach



    def sendEmail(self, message):
        """封装发送email方法"""
        try:

            smtpObj = smtplib.SMTP_SSL(self.sender_server, 465)
            smtpObj.connect(self.sender_server)

            smtpObj.login(self.sender, self.config['EMAIL']['Password'])
            smtpObj.sendmail(self.sender, self.receivers, message.as_string())
            smtpObj.quit()
            print("邮件发送成功")
        except smtplib.SMTPException as ex:
            print("Error: 无法发送邮件.%s"%ex)

    #发送调用
    def send(self):
        """发送email调用入口"""
        self.sendEmail(self.setMailContent)

if __name__ == "__main__":
    email = EmailClass()
    email.send()


#*_*coding:utf-8*_*
import xlrd as excel
import gc #垃圾回收
import os
from aceui.lib import gl
from xlutils.copy import copy




#Excel操作
class Excel(object):
    __slots__ = ['excelPath']

    def __init__(self,excelPath):
        self.excelPath =excelPath

    #创建workbook对象
    def OpenExcel(self,file='file.xls'):
        try:
            ret = excel.open_workbook(filename=self.excelPath)
            #data = excel.open_workbook(filename=self.excelPath)
            return  ret
        except Exception as ex:
            print(str(ex))

    #获取指定行数据,返回数组
    def getRowData(self,rownum =0):
        try:
            data = self.OpenExcel(self.excelPath)
            table = data.sheet_by_index(0)
        except Exception as ex:
            return ex.message()
        return table.row_values(rownum)


    def getCardNo(self,start_col=0,cell_col=1,cell_valueType=0,sheet_name='Sheet1'):
        """
        获取实体卡，卡号
        :param start_col: 定义列所在行号
        :param sheet_name: Sheet名称
        :return: 卡号或None
        """
        cardNo = None
        data = self.OpenExcel(self.excelPath)
        table = data.sheet_by_name(sheet_name)
        rowCount = table.nrows

        for n in range(1,rowCount):
            openCardflag = table.cell(n,2).value
            if str(openCardflag).upper() != 'Y':
                cardNo = str(table.cell(n,cell_col).value) #卡号
                newData = copy(data)
                ws = newData.get_sheet(0)
                ws.write(n,2,'Y')
                newData.save(self.excelPath)
                break

        if cell_valueType ==0:
            cardNo = cardNo[1:]

        return cardNo




    #根据sheet名获取所有行数据，数组返回
    def getExcelDataByName(self,start_col=4,sheet_name='Sheet1'):

        data = self.OpenExcel(self.excelPath)
        table = data.sheet_by_name(sheet_name)
        RowCount = table.nrows
        ColCount = table.ncols
        ColName = table.row_values(start_col) #获取行，列名行数据，返回数组
        #print ColName
        list=[] #存储行数据，内容字典对象
        #遍历，列名与行数据存储在字典中，并将字典对象存在list中

        for rowNum in range(start_col+1,RowCount):
            #row = table.row_values(rowNum)
            dict = {}

            if table.cell_value(rowNum,0) !='END':
                #print type(table.cell_value(rowNum, 0))
                for i in range(len(ColName)):
                    dict[ColName[i]]=table.cell_value(rowNum,i)
                list.append(dict)
            else:
                break
        gc.collect()
        return list





if __name__=="__main__":
    excelPath = os.path.join(gl.dataPath, 'posChargeCard.xls').decode('utf-8')
    a =  Excel(excelPath).getCardNo(cell_col=0,cell_valueType=1)
    print(int(float(a)))


#_*_coding:utf-8_*_
"""
用于存储全局变量及路径
"""
import os
import time



global libPath
global reportPath #报告路径
global casePath
global imgPath
global configPath
global dataPath
global curDate


# 取路径绝对路径
PATH =lambda p:os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        p)
)

'''全局变量'''
libPath = PATH(os.path.dirname(__file__)) #lib目录
# 报告目录，绝对路径
reportPath = os.path.join(
    PATH(os.path.dirname(libPath)),
    'report'
    )


# case目录，绝对路径
casePath = os.path.join(
    PATH(os.path.dirname(libPath)),
    'testCase'
)
# 存储截图目录，绝对路径
imgPath = os.path.join(
    PATH(reportPath),
    'images')

# 配置文件目录，绝对路径
configPath = os.path.join(
    PATH(os.path.dirname(libPath)),
    'config'
)

# 数据文件目录，绝对路径
dataPath = os.path.join(
    PATH(os.path.dirname(libPath)),
    'data'
)
# 配置文件，绝对路径
configFile = os.path.join(
    configPath,
    'config.yaml'
)

# 当前日期yyyy-mm-dd
curDate = time.strftime('%Y-%m-%d')




if __name__ == "__main__":
    print("lib路径:%s"%libPath)
    print("report路径:%s"%reportPath)
    print('testCase路径:{0}'.format(casePath))
    print('report/images路径:{0}'.format(imgPath))
    print('config路径:{0}'.format(configPath))
    print('data路径:{0}'.format(dataPath))




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aceui', 'aceui.base', 'aceui.lib']

package_data = \
{'': ['*'], 'aceui': ['config/*', 'whls/*']}

install_requires = \
['Pillow==5.3.0',
 'PyAutoIt==0.4',
 'PyYAML==5.1.1',
 'ddt==1.1.3',
 'requests==2.25.0',
 'selenium==3.141.0']

entry_points = \
{'console_scripts': ['ace = aceui.main:run_min', 'aceui = aceui.main:run_min']}

setup_kwargs = {
    'name': 'aceui',
    'version': '1.0.4',
    'description': '基于Selenium的UI自动化测试框架',
    'long_description': '\ufeff# AceUI\n\nAceUI是基于Selenium的UI自动化测试框架。\n\n\n\n\n\n\n  \n### 通过poetry工具打包\n\n- poetry build\n\n- poetry config repositories.testpypi https://pypi.org/project/aceui\n\n- poetry publish  输入pypi用户名和密码',
    'author': '天枢',
    'author_email': 'lengyaohui@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/HttpTesting/aceui',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
}


setup(**setup_kwargs)



# ########################################################
# 将根目录加入sys.path中,解决命令行找不到包的问题
import sys
import os
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.insert(0, rootPath)
# ########################################################

import time
import unittest
import argparse
from aceui.lib.emailstmp import EmailClass
from aceui.lib.config import CONF
from aceui.lib import (HTMLTESTRunnerCN, gl, scripts)
from aceui.lib.scripts import send_dding_msg
from aceui import __version__


def run_min():
    """
    Perfrom interface test entry.

    Args: None
    Usage:
        Command line execution.
    Return:
        There is no return.
    """

    # Takes the current path of the command line
    cur_dir = os.getcwd()
    os.chdir(cur_dir)

    parse = argparse.ArgumentParser(
        description='AceUI UI自动化测试框架',
        prog='AceUI'
        )
    parse.add_argument(
        "-v",
        "--version",
        action='version',
        version="%(prog)s {}".format(__version__),
        help='Framework version.'
        )
    parse.add_argument(
        "-f",
        "--file",
        nargs='+',
        default='',
        help='The file path; File absolute or relative path.'
        )
    parse.add_argument(
        "-d",
        "--dir",
        default='',
        help='The folder path; folder absolute or relative path.'
        )

    parse.add_argument(
        "-conf",
        "--config",
        nargs="+",
        default='',
        help='Basic setting of framework.'
        )

    # Command line arguments are assigned to varibales.
    args = parse.parse_args()
    files = args.file
    dirs = args.dir

    # 执行单个文件或指定目录执行
    if files:
        run(files, cur_dir)
    elif dirs:
        casedir = os.path.join(os.getcwd(), dirs)
        run(casedir, cur_dir)



def run(args, curdir):
    '''
    执行测试
    '''
    # scripts.remove_all_files(gl.imgPath)
    suite = unittest.TestSuite()

    if isinstance(args, list):
        for i in args:
            suite.addTest(
                unittest.defaultTestLoader.discover(curdir, pattern=i)
            )            
    else:
        suite.addTest(
            unittest.defaultTestLoader.discover(args, pattern='test*.py')
        )

    report_path = os.path.join(curdir, 'reports')
    if not os.path.exists(report_path):
        os.makedirs(report_path)

    filePath = os.path.join(report_path, 'Report.html')  # 确定生成报告的路径
    print(
        '测试报告生成路径:{}'.format(filePath)
    )

    # 配置文件
    conf = CONF.read(gl.configFile)

    with open(filePath, 'wb') as fp:
        runner = HTMLTESTRunnerCN.HTMLTestRunner(
            stream=fp,
            title= conf['REPORT']['title'],
            description= conf['REPORT']['description'],  # 不传默认为空
            tester= conf['REPORT']['tester']  # 测试人员名字，不传默认为小强
        )

        if conf['MESSAGE']['DING']:

            TMPL_MSG = '{}:★开始{}★'.format(
                time.strftime(r'%Y%m%d_%H%M%S', time.localtime(time.time())),
                conf['REPORT']['title']
            )
            send_dding_msg(conf['MESSAGE']['TOKEN'], TMPL_MSG)

        # 运行测试用例
        runner.run(suite)

        if conf['MESSAGE']['DING']:
            # TMPL_MSG = '''Pro商家后台UI自动化测试执行【已完成】:\n{}\n测试报告:http://60.205.217.8:5004/pos/pro_manage_web/report'''.format(runner.RESULT)
            TMPL_MSG = conf['MESSAGE']['MSG'].format(runner.RESULT)
            send_dding_msg(conf['MESSAGE']['TOKEN'], TMPL_MSG)

        if conf['EMAIL']:
            EmailClass().send()


if __name__=="__main__":
    run_min()

